/*****************************************************************************

				WWIV Version 4
                    Copyright (C) 1988-1993 by Wayne Bell

Distribution of the source code for WWIV, in any form, modified or unmodified,
without PRIOR, WRITTEN APPROVAL by the author, is expressly prohibited.
Distribution of compiled versions of WWIV is limited to copies compiled BY
THE AUTHOR.  Distribution of any copies of WWIV not compiled by the author
is expressly prohibited.


*****************************************************************************/



#include "vars.h"

#pragma hdrstop

#include <dir.h>


#define SETREC(i)  lseek(dlf,((long) (i))*((long)sizeof(uploadsrec)),SEEK_SET);

/* the archive type to use */
#define ARC_NUMBER 0


/* strings not to allow in a .zip file to extract from */
char *bad_words[] = {
  "COMMAND",
  "..",
  NULL,
};

long bad_filename(char *fn)
{
  int i;

  strupr(fn);
  for (i=0; bad_words[i]; i++) {
    if (strstr(fn,bad_words[i])) {
      outstr(get_string(840));
      pl(fn);
      return(1);
    }
  }
  if (!okfn(fn)) {
    outstr(get_string(840));
    pl(fn);
    return(1);
  }
  return(0);
}

/****************************************************************************/

typedef struct {
  unsigned char type;
  char name[13];
  long len;
  int date,time,crc;
  long size;
} arch;

int check_for_files_arc(char *fn)
{
  int f;
  char s[20];
  arch a;
  long l1,l;

  f=open(fn,O_RDONLY | O_BINARY);
  if (f>0) {
    l1=filelength(f);
    l=1;
    lseek(f,0L,SEEK_SET);
    read(f,&a,1);
    if (a.type!=26) {
      close(f);
      outstr(stripfn(fn));
      pl(get_string(841));
      return(1);
    }
    while (l<l1) {
      lseek(f,l,SEEK_SET);
      if ((read(f,&a,sizeof(arch)))==sizeof(arch)) {
	l += sizeof(arch);
	if (a.type==1) {
	  l-=4;
	  a.size=a.len;
	}
	if (a.type) {
	  l += a.len;
	  ++l;
	  strncpy(s,a.name,13);
	  s[13]=0;
	  if (bad_filename(s)) {
	    close(f);
	    return(1);
	  }
	} else
	  l=l1;
      } else {
	close(f);
        if (a.type!=0) {
          outstr(stripfn(fn));
          outstr(get_string(841));
          return(1);
        } else
          l=l1;
      }
    }
    
    close(f);
    return(0);
  }
  outstr(get_string(842));
  pl(stripfn(fn));
  return(1);
}


/****************************************************************************/

/* .ZIP structures and defines */
#define ZIP_LOCAL_SIG 0x04034b50
#define ZIP_CENT_START_SIG 0x02014b50
#define ZIP_CENT_END_SIG 0x06054b50

typedef struct {
  unsigned long         signature;    /* 0x04034b50 */
  unsigned short        extract_ver;
  unsigned short        flags;
  unsigned short        comp_meth;
  unsigned short        mod_time;
  unsigned short        mod_date;
  unsigned long         crc_32;
  unsigned long         comp_size;
  unsigned long         uncomp_size;
  unsigned short        filename_len;
  unsigned short        extra_length;
} zip_local_header;

typedef struct {
  unsigned long         signature;    /* 0x02014b50 */
  unsigned short        made_ver;
  unsigned short        extract_ver;
  unsigned short        flags;
  unsigned short        comp_meth;
  unsigned short        mod_time;
  unsigned short        mod_date;
  unsigned long         crc_32;
  unsigned long         comp_size;
  unsigned long         uncomp_size;
  unsigned short        filename_len;
  unsigned short        extra_len;
  unsigned short        comment_len;
  unsigned short        disk_start;
  unsigned short        int_attr;
  unsigned long         ext_attr;
  unsigned long         rel_ofs_header;
} zip_central_dir;


typedef struct {
  unsigned long         signature;    /* 0x06054b50 */
  unsigned short        disk_num;
  unsigned short        cent_dir_disk_num;
  unsigned short        total_entries_this_disk;
  unsigned short        total_entries_total;
  unsigned long         central_dir_size;
  unsigned long         ofs_cent_dir;
  unsigned short        comment_len;
} zip_end_dir;


int check_for_files_zip(char *fn)
{
  int f;
  long l,sig,len;
  zip_local_header zl;
  zip_central_dir zc;
  zip_end_dir ze;
  char s[161];

#define READ_FN(ln) {read(f,s,ln); s[ln]=0;}

  f=open(fn,O_RDONLY | O_BINARY);
  if (f>0) {
    l=0;
    len=filelength(f);
    while (l<len) {
      lseek(f,l,SEEK_SET);
      read(f,&sig, 4);
      lseek(f,l,SEEK_SET);
      switch(sig) {
        case ZIP_LOCAL_SIG:
          read(f,&zl, sizeof(zl));
          READ_FN(zl.filename_len);
          if (bad_filename(s)) {
            close(f);
            return(1);
          }
          l += sizeof(zl);
          l += zl.comp_size + zl.filename_len + zl.extra_length;
          break;
        case ZIP_CENT_START_SIG:
          read(f,&zc, sizeof(zc));
          READ_FN(zc.filename_len);
          if (bad_filename(s)) {
            close(f);
            return(1);
          }
          l += sizeof(zc);
          l += zc.filename_len + zc.extra_len;
          break;
        case ZIP_CENT_END_SIG:
          read(f,&ze, sizeof(ze));
          close(f);
          return(0);
        default:
          close(f);
          pl(get_string(843));
          return(1);
      }
    }
    close(f);
    return(0);
  }

  outstr(get_string(842));
  pl(stripfn(fn));
  return(1);
}

/****************************************************************************/

typedef struct {
  unsigned char   checksum;
  char            ctype[5];
  long            comp_size;
  long            uncomp_size;
  unsigned short  time;
  unsigned short  date;
  unsigned short  attr;
  unsigned char   fn_len;
} lharc_header;



int check_for_files_lzh(char *fn)
{
  long l1,l;
  int f,err=0;
  unsigned short crc;
  char s[256];
  char flag;
  lharc_header a;

  if ((f=open(fn,O_RDONLY|O_BINARY))<0) {
    outstr(get_string(842));
    pl(stripfn(fn));
    return(1);
  }
  l1=filelength(f);
  for(l=0;l<l1;l+=a.fn_len+a.comp_size+sizeof(lharc_header)+read(f,&crc,sizeof(crc))+1) {
    lseek(f,l,SEEK_SET);
    read(f,&flag,1);
    if (!flag) {
      l=l1;
      break;
    }
    if ((read(f,&a,sizeof(lharc_header)))!=sizeof(lharc_header)) {
      outstr(stripfn(fn));
      pl(get_string(844));
      err=1;
      break;
    }
    if (read(f,s,a.fn_len)!=a.fn_len) {
      outstr(stripfn(fn));
      pl(get_string(844));
      err=1;
      break;
    }
    s[a.fn_len]=0;
    if (bad_filename(s)) {
      err=1;
      break;
    }
  }
  close(f);
  return(err);
}
/****************************************************************************/

int check_for_files_arj(char *fn)
{
  int f,i;
  char s[256];
  long l1,l,l2;
  unsigned short sh;
  unsigned char s1;


  f=open(fn,O_RDONLY | O_BINARY);
  if (f>0) {
    l1=filelength(f);
    l=0;
    lseek(f,0L,SEEK_SET);
    while (l<l1) {
      lseek(f,l,SEEK_SET);
      i=read(f,&sh,2);
      if ((i!=2) || (sh!=0xea60)) {
        close(f);
        outstr(stripfn(fn));
        outstr(get_string(845));
        return(1);
      }
      l+=i+2;
      read(f,&sh,2);
      read(f,&s1,1);
      lseek(f,l+12,SEEK_SET);
      read(f,&l2,4);
      lseek(f,l+(long)s1,SEEK_SET);
      read(f,s,250);
      s[250]=0;
      if (strlen(s)>240) {
        close(f);
        outstr(stripfn(fn));
        outstr(get_string(845));
        return(1);
      }
      l += 4+(long) sh;
      lseek(f,l,SEEK_SET);
      read(f,&sh,2);
      l += 2;
      while ((l<l1) && (sh)) {
        l += 6+(long) sh;
        lseek(f,l-2,SEEK_SET);
        read(f,&sh,2);
      }
      l += l2;
      if (bad_filename(s)) {
        close(f);
        return(1);
      }
    }
    
    close(f);
    return(0);
  }
  outstr(get_string(842));
  pl(stripfn(fn));
  return(1);
}



/****************************************************************************/


typedef struct {
  char *arc_name;
  int (*func)(char *);
} arc_testers;

arc_testers arc_t[] = {
  {"ZIP", check_for_files_zip},
  {"ARC", check_for_files_arc},
  {"LZH", check_for_files_lzh},
  {"ARJ", check_for_files_arj},
  {NULL, NULL},
};



int check_for_files(char *fn)
{
  char *ss;
  int i;

  ss=strrchr(fn,'.');
  if (ss) {
    ss++;
    for (i=0; arc_t[i].arc_name; i++) {
      if (stricmp(ss,arc_t[i].arc_name)==NULL) {
        return(arc_t[i].func(fn));
      }
    }
  } else {
    /* no extension? */
    pl(get_string(846));
    return(1);
  }
  return(0);
}


void download_temp_arc(char *fn, int xfer)
{
  char s[81],s1[81];
  long numbytes;
  double d;
  int i,f,sent,abort;

  outstr(get_string(847));
  npr("%s.%s:\r\n\r\n", fn, syscfg.arcs[ARC_NUMBER].extension);
  if (xfer && !ratio_ok()) {
    pl(get_string(848));
    return;
  }
  sprintf(s1,"%s%s.%s",syscfg.tempdir,fn, syscfg.arcs[ARC_NUMBER].extension);
  f=open(s1,O_RDONLY | O_BINARY);
  if (f<0) {
    pl(get_string(849));
    nl();
    return;
  }
  numbytes=filelength(f);
  close(f);
  if (numbytes==0L) {
    pl(get_string(850));
    nl();
    return;
  }
  d=((double) (((numbytes)+127)/128)) *
    (1620.0) /
    ((double) (modem_speed));
  if (d<=nsl()) {
    outstr(get_string(851));
    pl(ctim(d));
    sent=0;
    abort=0;
    sprintf(s,"%s.%s",fn,syscfg.arcs[ARC_NUMBER].extension);
    send_file(s1,&sent,&abort,0,s,-1,numbytes);
    if (sent) {
      if (xfer) {
        ++thisuser.downloaded;
        thisuser.dk += ((numbytes+1023)/1024);
        nl();
        nl();
        outstr(get_string(782)); npr("%-6.3f\r\n",ratio());
      }
      sprintf(s1,get_stringx(1,45),(numbytes+1023)/1024, s);
      sysoplog(s1);
      if (useron)
        topscreen();
    }
  } else {
    nl();
    nl();
    pl(get_string(787));
    nl();
  }
}

void add_arc(char *arc, char *fn, int dos)
{
  char s[255], s1[81];

  sprintf(s1,"%s.%s",arc, syscfg.arcs[ARC_NUMBER].extension);
  get_arc_cmd(s,s1,2,fn);
  if (s[0]) {
    cd_to(syscfg.tempdir);
    outs(s);
    outs("\r\n");
    if (dos) {
      do_external(s,0);
    } else {
      set_protect(0);
      do_remote(s, 1);
      topscreen();
    }
    cd_to(cdir);
    sprintf(s,get_stringx(1,46),fn, s1);
    sysoplog(s);
  } else
    pl(get_string(852));
}

void add_temp_arc(void)
{
  char s[81],s2[81];
  int i;

  nl();
  pl(get_string(853));
  pl(get_string(854));
  prt(2,": ");
  input(s,12);
  if (!okfn(s))
    return;
  if (s[0]==0)
    return;
  if (strchr(s,'.')==NULL)
    strcat(s,".*");
  strcpy(s2,stripfn(s));
  for (i=0; i<strlen(s2); i++)
    if ((s2[i]=='|') || (s2[i]=='>') || (s2[i]=='<') || (s2[i]==';') || (s2[i]==' ') ||
        (s2[i]==':') || (s2[i]=='/') || (s2[i]=='\\'))
      return;
  add_arc("TEMP", s2, 1);
}

void del_temp(void)
{
  char s[81],s1[81];
  nl();
  prt(2,get_string(855));
  input(s1,12);
  if (!okfn(s1))
    return;
  if (s1[0]) {
    if (strchr(s1,'.')==NULL)
      strcat(s1,".*");
    remove_from_temp(s1,syscfg.tempdir, 1);
  }
}

void list_temp_dir(void)
{
  int i,i1,f1,abort;
  char s[81],s1[81];
  struct ffblk ff;
  uploadsrec u;

  sprintf(s1,"%s*.*",syscfg.tempdir);
  f1=findfirst(s1,&ff,0);
  nl();
  pl(get_string(856));
  nl();
  i1=0;
  abort=0;
  while ((f1==0) && (!hangup) && (!abort)) {
    strcpy(s,(ff.ff_name));
    align(s);
    sprintf(s1,"%12s  %-8ld",s,ff.ff_fsize);
    pla(s1, &abort);
    f1=findnext(&ff);
    i1=1;
  }
  if (!i1)
    pl(get_string(5));
  nl();
  if ((!abort) && (!hangup)) {
    npr("%s: %ldk.\r\n", get_string(857),(long) freek1(syscfg.tempdir));
    nl();
  }
}


void temp_extract(void)
{
  int i,i1,i2,i3,ok,abort,ok1;
  char s[255],s1[255],s2[81],s3[255],s4[129];
  uploadsrec u,u1;

  dliscan();
  nl();
  pl(get_string(858));
  nl();
  prt(2,get_string(44));
  input(s,12);
  if ((!okfn(s)) || (s[0]==0)) {
    closedl();
    return;
  }
  if (strchr(s,'.')==NULL)
    strcat(s,".*");
  align(s);
  i=recno(s);
  ok=1;
  while ((i>0) && (ok) && (!hangup)) {
    SETREC(i);
    read(dlf,(void *)&u,sizeof(uploadsrec));
    sprintf(s2,"%s%s",directories[udir[curdir].subnum].path,u.filename);
    get_arc_cmd(s1,s2,1,"");
    if ((s1[0]) && (exist(s2))) {
      nl();
      nl();
      abort=0;
      printinfo(&u,&abort);
      nl();
      cd_to(directories[udir[curdir].subnum].path);
      get_dir(s4,1);
      strcat(s4,stripfn(u.filename));
      cd_to(cdir);
      if (!check_for_files(s4)) {
        do {
          prt(2,get_string(859));
          input(s1,12);
          if (s1[0]==0)
            ok1=0;
          else
            ok1=1;
          if (!okfn(s1))
            ok1=0;
          if (strcmp(s1,"?")==0) {
            list_arc_out(stripfn(u.filename),directories[udir[curdir].subnum].path);
            s1[0]=0;
          }
          if (strcmp(s1,"Q")==0) {
            ok=0;
            s1[0]=0;
          }
          i2=0;
          for (i1=0; i1<strlen(s1); i1++)
            if ((s1[i1]=='|') || (s1[i1]=='>') || (s1[i1]=='<') || (s1[i1]==';') || (s1[i1]==' '))
              i2=1;
          if (i2)
            s1[0]=0;
          if (s1[0]) {
            if (strchr(s1,'.')==NULL)
              strcat(s1,".*");
            get_arc_cmd(s3,s4,1,stripfn(s1));
            cd_to(syscfg.tempdir);
            if (!okfn(s1))
              s3[0]=0;
            if (s3[0]) {
              do_external(s3,0);
              sprintf(s2,get_stringx(1,47),s1,u.filename);
            } else
              s2[0]=0;
            cd_to(cdir);
            if (s2[0])
              sysoplog(s2);
          }
        } while ((!hangup) && (ok) && (ok1));
      }
    } else
      if (s1[0]) {
        nl();
        pl(get_string(860));
        nl();
      }
    if (ok)
      i=nrecno(s,i);
  }
  closedl();
}

void list_temp_text(void)
{
  int i,i1,f1,ok,sent;
  char s[81],s1[81];
  struct ffblk ff;
  uploadsrec u;
  double percent;

  nl();
  prt(2,get_string(861));
  input(s,12);
  if (!okfn(s))
    return;
  if (s[0]) {
    if (strchr(s,'.')==NULL)
      strcat(s,".*");
    sprintf(s1,"%s%s",syscfg.tempdir,stripfn(s));
    f1=findfirst(s1,&ff,0);
    ok=1;
    nl();
    while ((f1==0) && (ok)) {
      sprintf(s,"%s%s",syscfg.tempdir,ff.ff_name);
      nl();
      outstr(get_string(862));
      pl(ff.ff_name);
      nl();
      ascii_send(s,&sent,&percent);
      if (sent) {
        sprintf(s,get_stringx(1,48),ff.ff_name);
        sysoplog(s);
      } else {
        sprintf(s,get_stringx(1,49),ff.ff_name,percent*100.0);
        sysoplog(s);
        ok=0;
      }
      f1=findnext(&ff);
    }
  }
}


void list_temp_arc(void)
{
  char s1[81],s2[81];

  sprintf(s1,"TEMP.%s",syscfg.arcs[ARC_NUMBER].extension);
  list_arc_out(s1,syscfg.tempdir);
  nl();
}




void temporary_stuff(void)
{
  char s[81],s1[81],ch;
  int done;

  done=0;
  do {
    nl();
    prt(2,get_string(863));
    ch=onek("Q?DRAVLT");
    switch(ch) {
      case 'Q':
	done=1;
	break;
      case 'D':
        download_temp_arc("TEMP", 1);
	break;
      case 'V':
	list_temp_arc();
	break;
      case 'A':
        add_temp_arc();
	break;
      case 'R':
	del_temp();
	break;
      case 'L':
	list_temp_dir();
	break;
      case 'T':
	list_temp_text();
	break;
      case '?':
	printmenu(14);
	break;
    }
  } while ((!hangup) && (!done));
}



